@@ -23,6 +23,8 @@ namespace eval SelectColor {
         \#ffffff \#ffffff \#ffffff \#ffffff \#ffffff
     }
 
+    # Namespace variables overwrite global variables in TCL8
+    # Not changed here, as fixed in TCL9
     if {[string equal $::tcl_platform(platform) "unix"]} {
         set useTkDialogue 0
     } else {
@@ -418,7 +420,7 @@ proc SelectColor::dialog {path args} {
     # (2) ::SelectColor::_entryColor is modified (except by the user typing in
     #     the entry widget)
 
-    trace add variable ::SelectColor::_unsavedSelection write ::SelectColor::_SetEntryValue
+    trace add variable _unsavedSelection write ::SelectColor::_SetEntryValue
 
     $top add -text [lindex [BWidget::getname ok] 0]
     $top add -text [lindex [BWidget::getname cancel] 0]
@@ -436,7 +438,7 @@ proc SelectColor::dialog {path args} {
         set color ""
     }
 
-    trace remove variable ::SelectColor::_unsavedSelection write ::SelectColor::_SetEntryValue
+    trace remove variable _unsavedSelection write ::SelectColor::_SetEntryValue
 
     destroy $top
     return $color
@@ -502,7 +504,7 @@ proc SelectColor::_select_rgb {count} {
 
         # Display selected color in entry widget (via trace on
         # ::SelectColor::_unsavedSelection), and notify caller.
-        set ::SelectColor::_unsavedSelection $bg
+        set _unsavedSelection $bg
         _userCommand $bg
     }
 }
@@ -520,7 +522,7 @@ proc SelectColor::_set_rgb {rgb} {
 
     # Display selected color in entry widget (via trace on
     # ::SelectColor::_unsavedSelection), and notify caller.
-    set ::SelectColor::_unsavedSelection $rgb
+    set _unsavedSelection $rgb
     _userCommand $rgb
     set user [expr {$_selection-[llength $_baseColors]}]
     if {$user >= 0} {
@@ -808,7 +810,10 @@ proc SelectColor::_SetEntryValue {argVarName var2 op} {
     variable _entryColor
     variable _unsavedSelection
 
-    if {[string equal $argVarName ::SelectColor::_unsavedSelection] &&
+    # get the full qualified name
+    set fqname [uplevel 1 [list namespace which -variable $argVarName]]
+
+    if {[string equal $fqname ::SelectColor::_unsavedSelection] &&
             [string equal $var2 {}] && [string equal $op "write"]} {
         # OK
     } else {
@@ -817,10 +822,10 @@ proc SelectColor::_SetEntryValue {argVarName var2 op} {
                 \"$argVarName\", \"$var2\", \"$op\""
     }
 
-    set col24bit [::SelectColor::_24BitRgb [set $argVarName]]
+    set col24bit [_24BitRgb [set $fqname]]
 
     if {[_ValidateColorEntry forced $col24bit]} {
-        set ::SelectColor::_entryColor $col24bit
+        set _entryColor $col24bit
     } else {
         # Value is invalid, and if written to _entryColor this would disable
         # validation.
@@ -871,31 +876,46 @@ proc SelectColor::_ValidateColorEntry {percentV percentP} {
     variable _unsavedSelection
 
     set result [regexp -- {^#[0-9a-fA-F]*$} $percentP]
-    set lenny  [string length $percentP]
-
     if {$result} {
-        if {[string equal $percentV "forced"]} {
-            # Validation only.  Don't want a loop.
-        } elseif {[string equal $percentV "key"]} {
-            # Copy to GUI if a valid color.
-            if {($lenny - 1) % 3 || $lenny == 1} {
-                # Not a valid color, which needs 3n+1 characters, n > 0
-            } else {
-                after idle [list SelectColor::_SetWithoutTrace $percentP]
-            }
-        } elseif {[string equal $percentV "focusout"]} {
-            # If the color is valid it will already have been copied to the GUI
-            # and to _userCommand by the "key" validation above.
-            #
-            # The code below only needs to reset the value in the entry widget.
-            # Remove an invalid value, convert a valid one to 24-bit.
-            # Ignore $percentP, just fire the trace on _unsavedSelection.
-            set color $_unsavedSelection
-            after idle [list set ::SelectColor::_unsavedSelection $color]
+        # Check for a valid rgb color, which needs 3n+1 characters, n > 0
+        set lenny  [string length $percentP]
+        set entryincomplete [expr {($lenny - 1) % 3 || $lenny == 1}]
+    } else {
+        # Check for named colors
+        set result [regexp -- {^[a-zA-Z0-9 ]*$} $percentP]
+        # We do not accept the key stroke
+        if {!$result} {
+            return 0
+        }
+        # Check for complete named color
+        set entryincomplete [catch {winfo rgb . $percentP} rgblist]
+        if {!$entryincomplete} {
+            set red [expr {[lindex $rgblist 0]/0x100}]
+            set green [expr {[lindex $rgblist 1]/0x100}]
+            set blue [expr {[lindex $rgblist 2]/0x100}]
+            set percentP [format "#%02X%02X%02X" $red $green $blue]
         }
     }
 
-    return $result
+    if {[string equal $percentV "forced"]} {
+        # Validation only.  Don't want a loop.
+    } elseif {[string equal $percentV "key"]} {
+        # Copy to GUI if a valid color.
+        if {!$entryincomplete} {
+            after idle [list SelectColor::_SetWithoutTrace $percentP]
+        }
+    } elseif {[string equal $percentV "focusout"]} {
+        # If the color is valid it will already have been copied to the GUI
+        # and to _userCommand by the "key" validation above.
+        #
+        # The code below only needs to reset the value in the entry widget.
+        # Remove an invalid value, convert a valid one to 24-bit.
+        # Ignore $percentP, just fire the trace on _unsavedSelection.
+        set color $_unsavedSelection
+        after idle [list set SelectColor::_unsavedSelection $color]
+    }
+
+    return 1
 }
 
 
@@ -908,11 +928,14 @@ proc SelectColor::_ValidateColorEntry {percentV percentP} {
 # ------------------------------------------------------------------------------
 
 proc SelectColor::_SetWithoutTrace {value} {
-    trace remove variable ::SelectColor::_unsavedSelection write ::SelectColor::_SetEntryValue
+    variable _hsv
+    variable _unsavedSelection
+
+    trace remove variable _unsavedSelection write ::SelectColor::_SetEntryValue
     _set_rgb $value
     set _hsv [eval rgbToHsv [winfo rgb . $value]]
     _set_hue_sat [lindex $_hsv 0] [lindex $_hsv 1]
     _set_value   [lindex $_hsv 2]
-    trace add variable ::SelectColor::_unsavedSelection write ::SelectColor::_SetEntryValue
+    trace add variable _unsavedSelection write ::SelectColor::_SetEntryValue
     return
 }
